@@ -8,10 +8,7 @@ use 5.010000;
 use Exporter qw( import );
 our @EXPORT_OK = qw( read_history_files add_uploaders_to_history choose_videos_from_saved_uploaders edit_sticky_file );
 
-use List::MoreUtils  qw( any none );
-use LWP::UserAgent   qw();
-use Term::ANSIScreen qw( :cursor :screen );
-use Term::Choose     qw( choose );
+use Term::Choose qw( choose );
 
 use App::YTDL::Helper qw( read_json write_json );
 
@@ -139,7 +136,6 @@ sub edit_sticky_file {
 }
 
 
-
 sub _available_uploader {
     my ( $sticky, $history, $removed_next ) = @_;
     my $choices = [ map( "* $_", @$sticky ) ];
@@ -196,46 +192,22 @@ sub _get_sticky_name_and_url {
 
 sub add_uploaders_to_history {
     my ( $set, $opt, $data ) = @_;
-    for my $ex ( keys %$data ) {
-        for my $video_id ( keys %{$data->{$ex}} ) {
-            my $uploader    = $data->{$ex}{$video_id}{uploader};
-            my $uploader_id = $data->{$ex}{$video_id}{uploader_id};
-            return if ! length $uploader_id;
-            my $uploader_url;
-            if ( $ex eq 'youtube' ) {
-                my $user_url    = sprintf 'https://www.youtube.com/user/%s', $uploader_id;
-                my $channel_url = sprintf 'https://www.youtube.com/channel/%s', $uploader_id;
-                return if any { $user_url eq $_ || $channel_url eq $_ } keys %{$set->{sticky}};
-                return if any { $user_url eq $_ || $channel_url eq $_ } keys %{$set->{history}};
-                my @type = ( length $uploader_id == 24 ? ( 'channel', 'user' ) : ( 'user', 'channel' ) );
-                my $ua = LWP::UserAgent->new(
-                    agent         => $opt->{useragent},
-                    timeout       => $opt->{timeout},
-                    show_progress => 1
-                );
-                $uploader_url = sprintf 'https://www.youtube.com/%s/%s', $type[0], $uploader_id;
-                my $res = $ua->head( $uploader_url );
-                print up( 1 ), cldown;
-                if ( ! $res->is_success ) {
-                    $uploader_url = sprintf 'https://www.youtube.com/%s/%s', $type[1], $uploader_id;
-                    my $res = $ua->head( $uploader_url );
-                    print up( 1 ), cldown;
-                    $uploader_url = undef if ! $res->is_success;
-                }
+    EXTRACTOR_KEY: for my $ex ( keys %$data ) {
+        UPLOADER_URL: for my $up ( keys %{$data->{$ex}} ) {
+            if ( $up eq 'unknown_uploader_url' ) {
+                return;
             }
-            elsif ( $ex eq 'vimeo' ) {
-                $uploader_url = 'https://vimeo.com/' . $uploader_id;
-            }
-            if ( defined $uploader_url ) {
-                if ( none{ $uploader_url eq $_ } keys %{$set->{sticky}} ) {
-                    $set->{history}{$uploader_url} = {
-                        uploader     => $uploader,
-                        uploader_id  => $uploader_id,
-                        extractor    => $ex,
-                        timestamp    => time(),
-                    };
-                }
+            if ( exists $set->{sticky}{$up} ) {
+                return;
             }
+            my @keys = ( keys %{$data->{$ex}{$up}} );
+            my $id = $keys[0];
+            $set->{history}{$up} = {
+                uploader    => $data->{$ex}{$up}{$id}{uploader},
+                uploader_id => $data->{$ex}{$up}{$id}{uploader_id},
+                extractor   => $ex, # extractor_key #
+                timestamp   => time(),
+            };
         }
     }
     my $c_hist = $set->{history};
